// file      : mapping/traits-sqlite.h
// copyright : not copyrighted - public domain

#ifndef TRAITS_SQLITE_H
#define TRAITS_SQLITE_H

//
// SQLite implementation.
//

#include <cstddef> // std::size_t
#include <cstring> // std::strncmp, std::memcpy
#include <iomanip>
#include <sstream>

#include <odb/sqlite/traits.hxx>

#include <QPointF>
#include <QVector3D>
#include <QImage>
#include <DirectBuffer.h>
#include <QColor>

namespace odb {
namespace sqlite {

template<>
class value_traits<QImage, id_blob> {
public:
  typedef QImage value_type;
  typedef QImage query_type;
  typedef details::buffer image_type;

  static void
  set_value(QImage &v,
            const details::buffer &b,
            std::size_t n,
            bool is_null) {
    using namespace std;

    if (!is_null)
      v = QImage::fromData((const uchar *)b.data(), n);
    else
      v = QImage();
  }

  static void
  set_image(details::buffer &b,
            std::size_t &n,
            bool &is_null,
            const QImage &v) {
    using namespace std;

    is_null = v.isNull();
    if(!is_null) {
      n = v.byteCount();

      if (n > b.capacity())
        b.capacity(n);

      lo::base::DirectBuffer buf((uchar *)b.data(), n);
      buf.open(QIODevice::WriteOnly);

      v.save(&buf, "PNG");
    }
  }
};

}
}


#endif // TRAITS_SQLITE_H
